/*This code shows how to make use of the DMC SCB priority feature to prioritize DMC transfers
 * for a specific master(e.g. MDMA).
 */

#include "main.h"
#include <sys/platform.h>
#include <sys/adi_core.h>
#include "time.h"

CGU_Settings CGU0_PARAM_LIST_DEFAULT;
DMC_PARAM DMC_Param_List;

#pragma section("seg_l1_block1")
uint32_t ucIntBuff1[16384];

#pragma section("seg_l1_block2")
uint32_t ucIntBuff2[16384];

volatile clock_t clock_start;
volatile clock_t clock_stop_mdma0;
volatile clock_t cycles_mdma0;
float throughput_mdma0;

volatile clock_t clock_stop_mdma1;
volatile clock_t cycles_mdma1;
float throughput_mdma1;

int DMA_Done_MDMA0=0;
int DMA_Done_MDMA1=0;

void MDMA0_Interrupt_Handler(uint32_t iid, void* handlerArg);
void MDMA1_Interrupt_Handler(uint32_t iid, void* handlerArg);

int main(void)
{
	int failure=0; int i; float fThroughput;

	//Initialize the DMC parameters list
	DMC_Param_List.dmc_no=0;
	DMC_Param_List.ddr_mode=DDR3_MODE;
	DMC_Param_List.phy_init_required=true;
	DMC_Param_List.cclk_dclk_ratio=1;
	DMC_Param_List.anomaly_20000037_applicable=true;
	DMC_Param_List.dmc_ctl_value=DMC_CTL_VALUE;
	DMC_Param_List.dmc_cfg_value=DMC_CFG_VALUE;
	DMC_Param_List.dmc_tr0_value=DMC_TR0_VALUE;
	DMC_Param_List.dmc_tr1_value=DMC_TR1_VALUE;
	DMC_Param_List.dmc_tr2_value=DMC_TR2_VALUE;
	DMC_Param_List.dmc_mr0_value=DMC_MR0_VALUE;
	DMC_Param_List.dmc_mr1_value=DMC_MR1_VALUE;
	DMC_Param_List.dmc_mr2_value=DMC_MR2_VALUE;
	DMC_Param_List.padctl2_value=DMC_PADCTL2_VALUE;
	DMC_Param_List.dmc_cphyctl_value=DMC_CPHYCTL_VALUE;
	DMC_Param_List.dmc_data_calib_add_value=DMC0_DATA_CALIB_ADD;

	//Initialize CGU
	CGUInit();

	//Initialize DMC0
	DMCInit(DMC_Param_List);

	DMC_Param_List.dmc_no=1;
	DMC_Param_List.dmc_data_calib_add_value=DMC1_DATA_CALIB_ADD;

	//Initialize DMC1
	DMCInit(DMC_Param_List);

	//Configuring sync mode
	*pREG_SCB1_MST00_SYNC=2;
	*pREG_SCB2_MST00_SYNC=2;

	//DMA work unit size = 2048 bytes
	unsigned int uiBytes=2048;
	unsigned int uiBytes1=2048;

	*pREG_SPU0_SECUREP88=0x3;
	*pREG_SPU0_SECUREP89=0x3;
	*pREG_SPU0_SECUREP90=0x3;
	*pREG_SPU0_SECUREP91=0x3;


	//*pREG_DMC0_PRIO=0x0008;	//DMA 8 high priority
	//*pREG_DMC0_PRIO=0x0208;		//DMA 18 high priority
	*pREG_DMC0_PRIOMSK=0xFFFF;

	//Initialize and map the interrupts here
	*pREG_SEC0_GCTL=ENUM_SEC_GCTL_EN;
	*pREG_SEC0_CCTL1=ENUM_SEC_CCTL_EN;
	adi_int_InstallHandler(INTR_MDMA0_DST, MDMA0_Interrupt_Handler, NULL, true);
	adi_int_InstallHandler(INTR_MDMA1_DST, MDMA1_Interrupt_Handler, NULL, true);

	*pREG_DMA8_BWMCNT=0x7FFFFFFF;
	*pREG_DMA18_BWMCNT=0x7FFFFFFF;

	//Configure MDMA0 (8-9) to read from DMC
	MDMA_CONFIG(8, SOURCE_DMA_CFG, 0x80000000, SOURCE_MSIZE, uiBytes/SOURCE_MSIZE,9,DEST_DMA_CFG, (uint32_t)ucIntBuff1|0x28000000, DEST_MSIZE, uiBytes/DEST_MSIZE);


	//Configure MDMA2 (18-19) to read from DMC
	MDMA_CONFIG(18, SOURCE_DMA_CFG, 0x81000000, SOURCE_MSIZE, uiBytes1/SOURCE_MSIZE,19,DEST_DMA_CFG, (uint32_t)ucIntBuff2|0x28000000, DEST_MSIZE, uiBytes1/DEST_MSIZE);


	//Enable the DMAs here
	*pREG_DMA8_CFG |= 0x1;
	*pREG_DMA9_CFG |= 0x1;
	*pREG_DMA18_CFG |= 0x1;
	*pREG_DMA19_CFG |= 0x1;

	//Start the cycle count
	clock_start = 0x7FFFFFFF;

	//Wait for the DMA done
	while(DMA_Done_MDMA0==0||DMA_Done_MDMA1==0);

	//Stop the cycle counts
	cycles_mdma0=clock_start-clock_stop_mdma0;
	cycles_mdma1=clock_start-clock_stop_mdma1;

	//Disable the DMAs
	DISABLE_DMAs(8,9);
	DISABLE_DMAs(18,19);

	//Calculate the throughput values
	throughput_mdma0=(float)(uiBytes*SCLK_SPEED/cycles_mdma0);
	throughput_mdma1=(float)(uiBytes*SCLK_SPEED/cycles_mdma1);
	printf("\n\n No of bytes=%d, No of cycles for MDMA0 =%d\n\n ",uiBytes,cycles_mdma0);
	printf("\n\n No of bytes=%d, No of cycles for MDMA1 =%d\n\n",uiBytes,cycles_mdma1);
	printf("Throughput MDMA0=%f MB/s, Throughput MDMA1=%f MB/s\n\n",throughput_mdma0, throughput_mdma1);
	return 0;

}

void CGUInit(void)
{
	int i;

	//PLLCLK = 450 MHz, CCLK = 450 MHz, SYSCLK = 225 MHz, DCLK = 450 MHz, SCLK0 = SCLK1 = 225 MHz
	CGU0_PARAM_LIST_DEFAULT.ctl_MSEL=18;
	CGU0_PARAM_LIST_DEFAULT.ctl_DF=0;
	CGU0_PARAM_LIST_DEFAULT.div_CSEL=1;
	CGU0_PARAM_LIST_DEFAULT.div_SYSSEL=2;
	CGU0_PARAM_LIST_DEFAULT.div_S0SEL=2;
	CGU0_PARAM_LIST_DEFAULT.div_S1SEL=2;
	CGU0_PARAM_LIST_DEFAULT.div_DSEL=1;

	//Set DMCx_PHY_CTL0.RESETDLL bit.
	*pREG_DMC0_PHY_CTL0=0x800;
	*pREG_DMC1_PHY_CTL0=0x800;

	//Initialize CGU to change the DCLK frequency
	CGU_Init(0, 0, &CGU0_PARAM_LIST_DEFAULT,  0);

	//Clear DMCx_PHY_CTL0.RESETDLL bit
	*pREG_DMC0_PHY_CTL0=0x000;
	*pREG_DMC1_PHY_CTL0=0x000;

	//Wait for 9000 DCLK cycles for the DLL to lock
	for(i=0;i<9000*DMC_Param_List.cclk_dclk_ratio;i++);
}

void MDMA0_Interrupt_Handler(uint32_t iid, void* handlerArg)
{
	//Stop the cycle count for MDMA0
	clock_stop_mdma0 =  *pREG_DMA8_BWMCNT_CUR;

	*pREG_DMA9_STAT |= ENUM_DMA_STAT_IRQDONE;			 //Clear the W1C interrupt status bit

	DMA_Done_MDMA0 = 1;
}

void MDMA1_Interrupt_Handler(uint32_t iid, void* handlerArg)
{
	//Stop the cycle count for MDMA1
	clock_stop_mdma1 = *pREG_DMA18_BWMCNT_CUR;

	*pREG_DMA19_STAT |= ENUM_DMA_STAT_IRQDONE;			 //Clear the W1C interrupt status bit

	DMA_Done_MDMA1 = 1;
}
